#!/usr/bin/env python

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY-menu
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# Small changes by Jani Jaakkola <jani.jaakkola@helsinki.fi>
# 18.10.2018

import os
import re
import gtk
import gio
import signal
import subprocess
import appindicator
import urllib
import pwd
import webbrowser

username = pwd.getpwuid(os.getuid()).pw_name
hostname = subprocess.check_output("hostname", shell=True).strip()

class IndicatorPlaces:
    BOOKMARKS_PATH = '/usr/share/hy-menu/hy-menu-items'

    def __init__(self):
        # self.ind = appindicator.Indicator("places", "/usr/share/hy-menu/hy-flame-cmyk.svg", appindicator.CATEGORY_APPLICATION_STATUS)
        # Fix for cubbli 18 mate. 
        self.ind = appindicator.Indicator("places", "/usr/share/hy-menu/hy-menu.png", appindicator.CATEGORY_APPLICATION_STATUS)
        self.ind.set_status(appindicator.STATUS_ACTIVE)
        if hostname.startswith('vdi-') or hostname.startswith('dx'):
            self.gpath = "/home/ad/P-drive"
            self.zpath = "/home/ad/home/" + username[:1] + "/" + username
        else:
            self.gpath = "sftp://pangolin.it.helsinki.fi/home/ad/P-drive"
            self.zpath = "sftp://pangolin.it.helsinki.fi/home/ad/home/" + username[:1] + "/" + username
        self.update_menu()
        return

    def create_menu_item(self, label, icon_name):
        image = gtk.Image()
        image.set_from_icon_name(icon_name, 5)

        item = gtk.ImageMenuItem()
        item.set_label(label)
        item.set_image(image)
        item.set_always_show_image(True)
        return item

    # Icons for different menu items
    def get_bookmark_icon(self, path):
        if path.endswith("helpdesk"):
            icon_name = "help"
        elif path.endswith("chat_index.html"):
            icon_name = "user-available-panel"
        elif path.startswith("vmware"):
            icon_name = "vmware-view"
        else:
            f = gio.File(path)
            try:
                info = f.query_info(gio.FILE_ATTRIBUTE_STANDARD_ICON)
                icon = info.get_icon()
                icon_name = icon.get_names()[0] if icon.get_names()[0] != '(null)' else 'folder'
            except (gio.Error, NameError):
                icon_name = "folder"

        return icon_name

    # This creates the menu
    def update_menu(self, widget = None, data = None):
        try:
            bookmarks = open(self.BOOKMARKS_PATH).readlines()
        except IOError:
            bookmarks = []

        # Create menu
        menu = gtk.Menu()
        self.ind.set_menu(menu)

        item = self.create_menu_item(hostname, "computer")
        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # Add bookmarks to the menu from the /usr/share/hy-menu/hy-menu-items
        for bm in bookmarks:
            path, label = bm.strip().partition(' ')[::2]

            if not label:
                label = os.path.basename(os.path.normpath(path))

            label = urllib.unquote(label)
            item = self.create_menu_item(label, self.get_bookmark_icon(path))
            item.connect("activate", self.on_bookmark_click, path)

            # Append the item to menu
            # Check if the host is vdi machine --> disable vmware items.
            if hostname.startswith('vdi-') and path.startswith('vmware-view'):
                continue
            else:
                menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # Computer info
        item = self.create_menu_item("Computer Info", "info")
        item.connect("activate", self.computerInfo)
        menu.append(item)

        # User info
        item = self.create_menu_item("User Info", "info")
        item.connect("activate", self.userInfo)
        menu.append(item)

        # HY install printers
        item = self.create_menu_item("HY Install Printers", "stock_print")
        item.connect("activate", self.InstallPrinter)
        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # HY install software
#        item = self.create_menu_item("HY Install Software", "emblem-downloads")
#        item.connect("activate", self.InstallSoftware)
#        menu.append(item)

        # Create group directory menu item
        item = self.create_menu_item("IT Department group directories", "network")
        item.connect("activate", self.on_netdrive_click, self.gpath)
        menu.append(item)

        # Create Z-drive menu item
        item = self.create_menu_item("Z-drive", "network")
        item.connect("activate", self.on_netdrive_click, self.zpath)
        menu.append(item)

        # Show the menu
        menu.show_all()
        return

    # Open network drive with the default file browser
    def on_netdrive_click(self, widget, path):
        subprocess.Popen('gtk-launch $(xdg-mime query default inode/directory|sed s/\\.desktop//) ' + path, shell = True)
        return

    # Open clicked bookmark
    def on_bookmark_click(self, widget, path):
        if path.endswith("chat_index.html"):
            subprocess.Popen('/usr/bin/chromium-browser --window-size=420,540 --app=' + path, shell=True)
        else:
            webbrowser.open(path, new=2, autoraise=True)
        return

    def userInfo(self, item):
        subprocess.Popen('/usr/share/hy-menu/userinfo.py %s', shell = True)
        return

    def computerInfo(self, item):
        subprocess.Popen('/usr/share/hy-menu/computerinfo.py %s', shell = True)
        return

    def InstallPrinter(self, item):
        subprocess.Popen('/usr/share/hy-menu/hy-printers.py %s', shell = True)
        return

#    def InstallSoftware(self, item):
#        subprocess.Popen('/usr/bin/hy-software.py %s', shell = True)
#        return

    def on_bookmarks_changed(self, filemonitor, file, other_file, event_type):
        if event_type == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
            print 'Bookmarks changed, updating menu...'
            self.update_menu()
        return

if __name__ == "__main__":
    # Catch CTRL-C
    signal.signal(signal.SIGINT, lambda signal, frame: gtk.main_quit())

    # Run the indicator
    i = IndicatorPlaces()

    # Monitor bookmarks changes
    file = gio.File(i.BOOKMARKS_PATH)
    monitor = file.monitor_file()
    monitor.connect("changed", i.on_bookmarks_changed)

    # Main gtk loop
    gtk.main()
